
#############################################################
#                                                           #
#                                                           #
#                                                           #
#       mathLib funcionality tests                          #
#                                                           #
#       Authors:    xzacekp00                               #
#                   xvalehk00                               #
#                   xdrapet00                               #
#                                                           #
#       Last update: 2024-03-18                             #
#                                                           #
#                                                           #
#                                                           #
#                                                           #
#                                                           #
#############################################################


import pytest
import math



import mathLib


# Run using:
#   
#   pytest --verbose calc_test.py


########################### Basic functions ###############################


class TestAddition:
    def test_positive_numbers(self):
        assert mathLib.add(3, 5) == 8
        assert mathLib.add(100, 200) == 300
        assert mathLib.add(123456789, 987654321) == 1111111110

    def test_negative_numbers(self):
        assert mathLib.add(-3, -5) == -8
        assert mathLib.add(-100, -200) == -300
        assert mathLib.add(-123456789, -987654321) == -1111111110

    def test_mixed_numbers(self):
        assert mathLib.add(-3, 5) == 2
        assert mathLib.add(100, -200) == -100
        assert mathLib.add(-123456789, 987654321) == 864197532

    def test_zero(self):
        assert mathLib.add(0, 5) == 5
        assert mathLib.add(0, -100) == -100
        assert mathLib.add(0, 0) == 0

    def test_floats(self):
        assert mathLib.add(2.5, 3.5) == 6.0
        assert mathLib.add(-2.5, 4.5) == 2.0
        assert mathLib.add(10.75, 0.25) == 11.0


class TestSubtraction:
    def test_positive_numbers(self):
        assert mathLib.sub(10, 3) == 7
        assert mathLib.sub(105, 20) == 85
        assert mathLib.sub(1000, 777) == 223

    def test_negative_numbers(self):
        assert mathLib.sub(-8, -15) == 7
        assert mathLib.sub(-200, -300) == 100
        assert mathLib.sub(-987, -1234) == 247

    def test_mixed_numbers(self):
        assert mathLib.sub(-3, 5) == -8
        assert mathLib.sub(30, -20) == 50
        assert mathLib.sub(-300, 200) == -500

    def test_zero(self):
        assert mathLib.sub(5, 0) == 5
        assert mathLib.sub(-100, 0) == -100
        assert mathLib.sub(0, 0) == 0

    def test_floats(self):
        assert mathLib.sub(3.5, 2.5) == 1.0
        assert mathLib.sub(4.7, -2.5) == 7.2
        assert mathLib.sub(10.75, 0.25) == 10.5


class TestMultiplication:
    def test_positive_numbers(self):
        assert mathLib.mul(5, 3) == 15
        assert mathLib.mul(10, 20) == 200
        assert mathLib.mul(123456, 789) == 97406784

    def test_negative_numbers(self):
        assert mathLib.mul(-5, -3) == 15
        assert mathLib.mul(-10, -20) == 200
        assert mathLib.mul(-123456, -789) == 97406784

    def test_mixed_numbers(self):
        assert mathLib.mul(-5, 3) == -15
        assert mathLib.mul(10, -20) == -200
        assert mathLib.mul(-123456, 789) == -97406784

    def test_zero(self):
        assert mathLib.mul(5, 0) == 0
        assert mathLib.mul(-100, 0) == 0
        assert mathLib.mul(0, 0) == 0

    def test_floats(self):
        assert mathLib.mul(2.5, 3) == 7.5
        assert mathLib.mul(-2.5, 4) == -10.0
        assert mathLib.mul(10.75, 0.25) == 2.6875


class TestDivision:
    def test_positive_numbers(self):
        assert mathLib.div(10, 2) == round(10/2,10)
        assert mathLib.div(100, 20) == round(100/20,10)
        assert mathLib.div(123456, 789) == round(123456/789,10)

    def test_negative_numbers(self):
        assert mathLib.div(-10, -2) == round(-10/-2,10)
        assert mathLib.div(-100, -20) == round(-100/-20,10)
        assert mathLib.div(-123456, -789) == round(-123456/-789,10)

    def test_mixed_numbers(self):
        assert mathLib.div(-10, 2) == round(-10/2,10)
        assert mathLib.div(100, -20) == round(100/-20,10)
        assert mathLib.div(-123456, 789) == round(-123456/789,10)

    def test_floats(self):
        assert mathLib.div(2.5, 0.5) == 5.0
        assert mathLib.div(-2.5, 4) == -0.625
        assert mathLib.div(10.75, 0.25) == 43.0

    def test_div_by_zero(self):
        with pytest.raises(ValueError):
            mathLib.div(5, 0)

        with pytest.raises(ValueError):
            mathLib.div(-100, 0)

        with pytest.raises(ValueError):
            mathLib.div(0, 0)


########################### More complex functions ###############################


class TestPowerOf:
    def test_positive_exponents(self):
        assert mathLib.powr(2, 3) == 8
        assert mathLib.powr(3, 4) == 81
        assert mathLib.powr(5, 2) == 25

    def test_zero_exponent(self):
        assert mathLib.powr(10, 0) == 1

    def test_negative_exponent(self):
        assert mathLib.powr(2, -3) == 1/2**3


class TestSquareRoot:
    def test_positive_numbers(self):
        assert mathLib.sqrt(9,2) == 3.0
        assert mathLib.sqrt(16,2) == 4.0
        assert mathLib.sqrt(125,3) == 5.0
        assert mathLib.sqrt(16,4) == 2.0


    def test_zero(self):
        assert mathLib.sqrt(0,2) == 0.0

    def test_negative(self):
        with pytest.raises(ValueError):
            mathLib.sqrt(-5,2)


class TestFactorial:
    def test_positive_numbers(self):
        assert mathLib.fact(5) == 120
        assert mathLib.fact(10) == 3628800
        assert mathLib.fact(20) == 2432902008176640000

    def test_zero(self):
        assert mathLib.fact(0) == 1

    def test_negative(self):
        with pytest.raises(ValueError):
            mathLib.fact(-5)


class TestLogarithm:
    def test_negative_input(self):
        with pytest.raises(ValueError):
            mathLib.log(0)
        with pytest.raises(ValueError):
            mathLib.log(-1)
        with pytest.raises(ValueError):
            mathLib.log(mathLib.sub(1, 100))

    def test_positive_input(self):
        assert mathLib.log(1) == pytest.approx(0.0)
        assert mathLib.log(10) == pytest.approx(math.log(10), rel=1e-5)
        assert mathLib.log(100) == pytest.approx(math.log(100), rel=1e-5)

    def test_accuracy(self):
        assert mathLib.log(2) == pytest.approx(0.6931471806, rel=1e-5)
        assert mathLib.log(3) == pytest.approx(1.0986122887, rel=1e-5)


class TestAbsoluteValue:
    def test_positive_numbers(self):
        assert mathLib.abs(5) == 5
        assert mathLib.abs(10) == 10
        assert mathLib.abs(123456789) == 123456789

    def test_negative_numbers(self):
        assert mathLib.abs(-5) == 5
        assert mathLib.abs(-10) == 10
        assert mathLib.abs(-123456789) == 123456789

    def test_zero(self):
        assert mathLib.abs(0) == 0


########################### EOF test.py ###############################
