## @file mathLib.py
#  @brief Custom simple math library for 2nd IVS project 
#  @author xtichyt00
#  @date March 25, 2024

## Adds two numbers.
#  @param x The first number.
#  @param y The second number.
#  @return The sum of x and y.
def add(x, y):
    return round(x + y, 10)

## Subtracts two numbers.
#  @param x The first number.
#  @param y The second number.
#  @return The difference between x and y.
def sub(x, y):
    return round(x - y, 10)

## Multiplies two numbers.
#  @param x The first number.
#  @param y The second number.
#  @return The product of x and y.
def mul(x, y):
    return round(x * y, 10)

## Divides the first number by the second and returns the quotient.
#  @param x The dividend.
#  @param y The divisor.
#  @return The quotient of x divided by y.
#  @exception ValueError if the divisor is zero.
def div(x, y):
    if y == 0:
        raise ValueError("Cannot divide by zero.")
    return round(x / y, 10)

## Calculates power of a number.
#  @param x The number to find the power
#  @param y The power
#  @return The square root of x.
#  @exception ValueError if x is negative.
def powr(x, y):
    return round(x ** y, 10)

## Calculates the square root of a number.
#  @param x The number to find the square root of, must be non-negative.
#  @return The square root of x.
#  @exception ValueError if x is negative.
def sqrt(x, y):
    if x < 0:
        raise ValueError("Cannot calculate the square root of a negative number.")
    return round(x ** (1/y), 10)

## Calculates the factorial of a number.
#  @param n The number to calculate the factorial for, must be a non-negative integer.
#  @return The factorial of n.
#  @exception ValueError if n is negative or not an integer.
def fact(n):
    if n < 0:
        raise ValueError("Cannot calculate factorial of negative numbers.")
    elif not isinstance(n, int):
        raise ValueError("Factorial function requires an integer input.")
    result = 1
    for i in range(1, n + 1):
        result *= i
    return result

## Calculates the natural logarithm of a number using an iterative approach.
#  @param x The number to calculate the logarithm for, must be positive.
#  @return The natural logarithm of x.
#  @exception ValueError if x is non-positive.
def log(x):
    if x <= 0:
        raise ValueError("Logarithm undefined for non-positive values.")
    n = 1000000  # Number of iterations for approximation
    return round(n * ((x ** (1/n)) - 1), 10)

## Calculates the absolute value of a number.
#  @param x The number to calculate the absolute value for.
#  @return The absolute value of x.
def abs(x):
    return round(x if x >= 0 else -x, 10)

