
##
# @file deviation.py
# @brief Program for standard deviation calculation for profiling.
# @author xvalehk00
# @date April 6, 2024

from mathLib import *
from fileinput import input
import cProfile

##
# @brief Sums numbers from given list.
# @param val List of values to be summed.
# @return Total sum of all numbers from list.
#
def sum_numbers(val):
    sum_total = 0
    for n in val:
        sum_total = add(sum_total, n)
    return sum_total


##
# @brief Checks if the count of numbers is at least 2.
# @param numbers List of numbers.
#
def check_numbers_count(numbers):
    n = len(numbers)
    if n < 2:
        raise ValueError("Invalid input data (numbers count must be greater than 2).\n")


##
# @brief Calculates arithmetic mean value.
# @param calc_sum Summary of numbers to calculate mean from.
# @param leng Amount of numbers to calculate mean from.
# @return Arithmetic mean of given summary of given amount of numbers.
#
def mean(calc_sum, leng):
    return mul(div(calc_sum, leng), 1.0)


##
# @brief Calculates standard deviation from the list.
# @param leng Amount of numbers in the list.
# @param val List of values for standard deviation calculation.
# @return Standard deviation of numbers from given list.
#
def deviation(leng, val):
    sum_total = sum_numbers(val)
    mean_value = mean(sum_total, leng)
    sum_squared_deviations = 0
    for n in val:
        sum_squared_deviations = add(sum_squared_deviations, pow(sub(n, mean_value), 2))
    sum_squared_deviations = mul(sum_squared_deviations, div(1, sub(leng, 1)))
    return sqrt(sum_squared_deviations)


##
# @brief Loads numbers from standard input and prints their standard deviation.
# @param numbers List of numbers.
#


def main(numbers):
    try:
        values = list()
        for line in numbers:
            values.append(float(line))
        check_numbers_count(values)
        print(deviation(len(values), values))
        #cProfile.runctx('deviation(len(values), values)', globals(), locals())
    except ValueError as ve:
        print("Error:", ve)



main(input())


##
# @}
#

